//
// Copyright (c) 2012, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using MindFusion.Diagramming;
using MindFusion.Drawing;


namespace MindFusion.Diagramming.WinForms.Samples.CS.LogicGates
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
            diagram.DiagramLinkStyle.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(0, 52, 102));
            diagram.DiagramLinkStyle.HeadStroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(0, 52, 102));
		}

		private void mitNew_Click(object sender, System.EventArgs e)
		{
			if (MessageBox.Show("Are You Sure", "Confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
			{
				diagram.ClearAll();
				labelCounter = 0;
				counter = -1;
			}
		}

		private void mitSave_Click(object sender, System.EventArgs e)
		{
			ofd.InitialDirectory = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			if (sfd.ShowDialog(this) == DialogResult.OK)
				diagram.SaveToFile(sfd.FileName, true);
		}

		private void mitLoad_Click(object sender, System.EventArgs e)
		{
			ofd.InitialDirectory = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			if (ofd.ShowDialog(this) == DialogResult.OK)
			{
				try
				{
					diagram.LoadFromFile(ofd.FileName);
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.Message);
				}
			}
		}

		private void mitExit_Click(object sender, System.EventArgs e)
		{
			Application.Exit();
		}

		private void mitGenerateTruthTable_Click(object sender, System.EventArgs e)
		{
			dT = null;
			dT = clsBLL.BuildTransactionTable();


			foreach (ShapeNode bo in diagram.Nodes)
			{
				string[] s = bo.Tag.ToString().Split('|');
				DiagramLinkCollection iac = bo.IncomingLinks;
				DiagramLinkCollection oac = bo.OutgoingLinks;

				if (clsBLL.IsBoxProperlyLinked(Convert.ToInt32(s[0]), iac.Count, oac.Count) == true)
				{
					DataRow r = dT.NewRow();
					if (iac.Count == 0)
					{
						r["FirstSourceID"] = -15;
						r["SecondSourceID"] = -15;
						r["FirstSourceType"] = -15;
						r["SecondSourceType"] = -15;
					}
					else if (iac.Count == 1)
					{
						string[] r1 = iac[0].Origin.Tag.ToString().Split('|');
						r["FirstSourceID"] = r1[1];
						r["SecondSourceID"] = -15;
						r["FirstSourceType"] = r1[0];
						r["SecondSourceType"] = -15;
					}
					else if (iac.Count == 2)
					{
						string[] r2 = iac[0].Origin.Tag.ToString().Split('|');
						string[] r3 = iac[1].Origin.Tag.ToString().Split('|');
						r["FirstSourceID"] = r2[1];
						r["SecondSourceID"] = r3[1];
						r["FirstSourceType"] = r2[0];
						r["SecondSourceType"] = r3[0];
					}

					if (oac.Count == 0)
					{
						r["DestinationID"] = -15;
						r["DestinationType"] = -15;
					}
					else if (oac.Count == 1)
					{
						string[] r4 = oac[0].Destination.Tag.ToString().Split('|');
						r["DestinationID"] = r4[1];
						r["DestinationType"] = r4[0];
					}

					r["CurrentID"] = s[1];
					r["CurrentType"] = s[0];
					r["OutputResult"] = -15;
					r["FirstInput"] = -15;
					r["SecondInput"] = -15;
					r["BoxText"] = bo.Text;

					int boxNodeType = Convert.ToInt32(s[0]);
					if (boxNodeType==8 || boxNodeType==9)
					{

						r["StarterID"]= bo.Text.Substring(0,1);
					}
					if (boxNodeType == 7 ) //circuit
					{
						try
						{
							circuits.LoadFromFile(bo.Text);
						}
						catch (System.IO.FileNotFoundException)
						{
							MessageBox.Show("Please Select a File for the Circuit");
							return;
						}
						int circuitBoxResult;
						circuitBoxResult = clsBLL.circuitCalculator(ref circuits);
						r["OutputResult"] = circuitBoxResult;
					}
                    
					dT.Rows.Add(r);
				}
				else
				{
					MessageBox.Show("Incorrect Circuit, Please check links");
					return;
				}

			} //end of for
  
			//clsBLL.BuildOutput(ref dT);
			int finalResult= clsBLL.CircuitResultBuilder(ref dT);

			ResultForm f = new ResultForm();
			//result
			f.SetResult(finalResult.ToString());

			//truth table
			DataTable truthDataTable;
			truthDataTable = null;
			truthDataTable=  clsBLL.BuildTruthTable();
			int InputsCounts=  clsBLL.TruthTablePrepare(ref dT, ref truthDataTable);
			f.tt = clsBLL.GeneratedTruthTable; //truthDataTable;
			f.SetData(f.tt);
			f.dT = truthDataTable;
			f.inputsCount = InputsCounts;
			f.ShowDialog();
		}

		private void MainForm_Load(object sender, System.EventArgs e)
		{
			labelCounter = 1;
          
			lsvToolbox.Items[0].Selected = true;
            
			view.ScrollTo(0, 0);

			showInstrctionBox();

			counter = -1; //to number nodes
		}
	
		
		private bool notDuplicateTermainteNode(int type)
		{
			if (type == 10)
			{
				foreach (ShapeNode b in diagram.Nodes)
				{
					string[] s = b.Tag.ToString().Split('|');
					if (s.Length > 0)
					{
						int intS = Convert.ToInt32(s[0]);
						if (intS == 10)
							return false;
					}
				}
				return true;
			}
			else
			{
				return true;
			}
		}
	
		
     
		//supportive code
		private void showInstrctionBox()
		{

			ShapeNode ib = new ShapeNode(diagram);
			ib.Bounds = new RectangleF(10, 10, 250, 50);
			ib.Text = "To create a node, select node type from the list on the left, then " +
				"single-click in the drawing surface on the right.";
			ib.Tag = 42;
			ib.Brush = new MindFusion.Drawing.LinearGradientBrush(
                Color.White, Color.FromArgb(156, 170, 198), 30);
			diagram.Nodes.Add(ib);
		}


		private void createNode(int type, float x, float y)
		{

			counter += 1;
			ShapeNode node = diagram.Factory.CreateShapeNode(x - 16, y - 16, 70, 70);
			node.Transparent = true;
			node.EnabledHandles = AdjustmentHandles.Move; // no resizing
			node.Tag = type + "|" + counter.ToString();

			node.ImageAlign = ImageAlign.TopLeft;
			node.TextFormat.Alignment = StringAlignment.Center;
			node.TextFormat.LineAlignment = StringAlignment.Far;
			node.AnchorPattern = clsBLL.CreatePattern(type);

		
			switch (type)
			{
				case 0: //and
				{
					node.Image = ilsIcons.Images[0];
					node.Text = "AND";
					break;
				}
				case 1: //nand
				{
					node.Image = ilsIcons.Images[1];
					node.Text = "NAND";
					break;
				}
				case 2: //nor
				{
					node.Image = ilsIcons.Images[2];
					node.Text = "NOR";
					break;
				}
				case 3: //not
				{
					node.Image = ilsIcons.Images[3];
					node.Text = "NOT";
					break;
				}
				case 4: //or
				{
					node.Image = ilsIcons.Images[4];
					node.Text = "OR";
					break;
				}
				case 5: //xnor
				{
					node.Image = ilsIcons.Images[5];
					node.Text = "XNOR";
					break;
				}
				case 6: //xor
				{
					node.Image = ilsIcons.Images[6];
					node.Text = "XOR";
					break;
				}
				case 7: //circuit
				{
					node.Image = ilsIcons.Images[7];
					node.Text = "CIRCUIT";
					break;
				}
				case 8: //start0
				{
					node.Image = ilsIcons.Images[8];
					node.Text = labelCounter + ":START0";
					labelCounter += 1;
					break;
				}
				case 9: //start1
				{
					node.Image = ilsIcons.Images[9];
					node.Text = labelCounter + ":START1";
					labelCounter += 1;
					break;
				}
				case 10: //terminator
				{
					node.Image = ilsIcons.Images[10];
					node.Text = "TERMINATOR";
					break;
				}
			}

		}
	
		//local variables
		DataTable dT;
		int counter;
		int labelCounter;
		
		private void diagram_Clicked(object sender, MindFusion.Diagramming.DiagramEventArgs e)
		{
			if (e.MouseButton == MouseButton.Left)
			{
				if (lsvToolbox.SelectedItems.Count == 0)
					return;
				if (notDuplicateTermainteNode(lsvToolbox.SelectedItems[0].Index) == true)
					createNode(lsvToolbox.SelectedItems[0].Index, e.MousePosition.X, e.MousePosition.Y);
				else
					MessageBox.Show("There is already a terminator in the circuit");

				//delete the instruction box
				ShapeNode ib = diagram.FindNode(42) as ShapeNode;
				if (ib != null)
					diagram.Nodes.Remove(ib);
			}
			else
			{
				diagram.Selection.Clear();
				
				Point point = view.DocToClient((e.MousePosition));
				point.X += view.Bounds.X;
				cxmRightClick.Show(this, point);
			}
		}

		private void diagram_NodeClicked(object sender, MindFusion.Diagramming.NodeEventArgs e)
		{
			ShapeNode node = e.Node as ShapeNode;
			if (node == null)
				return;

			//let the user choose a file for the circuit
			string[] s = node.Tag.ToString().Split('|');
			if (s.Length > 0)
			{
				int intS = Convert.ToInt32(s[0]);
				if (intS == 7)
					if (ofd.ShowDialog() == DialogResult.OK)
						node.Text = ofd.FileName;
			}
		}

		private void diagram_NodeCreating(object sender, MindFusion.Diagramming.NodeValidationEventArgs e)
		{
			e.Cancel = true;
		}
	}
}
